from api_manager.async_response.lazy_request import lazy_request
from api_manager.constants import Project, APIState, Entities
from api_manager.methods import get, post, put, patch, delete
from api_manager.token import get_alms_token, get_labs_token

__all__ = [
    'get_alms_token',
    'get_labs_token',
    'get',
    'post',
    'put',
    'patch',
    'delete',
    'lazy_request',
    'Project',
    'APIState',
    'Entities'
]



from requests import request

from api_manager.constants import Project
from api_manager.exceptions import NotConfiguredException
from api_manager.headers import HeadersTemplates, PROJECT_HEADERS
from api_manager.logger import logging_response


def basic_request(
        method: str,
        url: str,
        user: dict = None,
        with_auth: bool = True,
        headers_type: HeadersTemplates = HeadersTemplates.DEFAULT,
        project: Project = None,
        **kwargs
):
    """
    :param method: Http method, like GET, POST, PUT, PATCH, DELETE
    :param url: Endpoint url
    :param project:
    :param user: user which should be used for getting ``Bearer`` token.
    If user is None then by default ``DEFAULT_USER`` is used. If you want to
    use other user, then provide credentials as:
    {
        "username": <some_email>,
        "password": <some_password>,
        "client_id": "insomnia",
        "client_secret": "insomnia",
        "grant_type": "password"
    }
    :param with_auth: FLag that declare should we use authentication headers for request either not.
    If set to True, then request will have header like:
    {'Authorization': 'Bearer <some_token>'}
    :param headers_type: Headers type which should be used for request. We need this because,
    some of requests should have for example {'Content-type': 'application/json'}, but
    others do not need any headers.

    All available headers can be viewed in ``HeadersTemplates``. If you want to add your
    custom headers, then add new headers to ``HeadersTemplates``, and pass it inside
    method like this:
    post(<url>, data={}, headers_type=HeadersTemplates.MY_CUSTOM)

    :param kwargs: additional params
    :return: :class:`Response <Response>` object

    This method has included logger, if you want to stop seeing API logs,
    then in settings.py set ``REQUESTS_LOGGING`` to False.

    ``Verify`` property by default set in setting.py ``CERT_PATH``. We need
    this property because if not set, then we will unable to make request.
    If set to False, then we will get insecure warning, but if set to
    some cert path, then no warnings and no errors.
    """
    from api_manager.settings import PROJECT, CERT_PATH

    safe_project = project or PROJECT

    if safe_project is None:
        raise NotConfiguredException(
            'You should define "PROJECT" setting in your settings file.'
            'Or provide "project" attribute to your method'
        )

    kwargs.setdefault('headers', PROJECT_HEADERS[safe_project](user, with_auth, headers_type))
    if CERT_PATH:
        kwargs.setdefault('verify', CERT_PATH)

    response = request(method, url, **kwargs)
    logging_response(response)

    return response


def get(url, params=None, user=None, **kwargs):
    """
    Sends a GET request.

    :param user: user to get token
    :param url: URL for the new :class:`Request` object.
    :param params: (optional) Dictionary, list of tuples or bytes to send
            in the query string for the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """
    return basic_request('get', url, user=user, params=params, **kwargs)


def post(url, json=None, user=None, **kwargs):
    """
    Sends a POST request.

    :param user: user to get token
    :param url: URL for the new :class:`Request` object.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """
    return basic_request('post', url, user=user, json=json, **kwargs)


def put(url, json=None, user=None, **kwargs):
    """
    Sends a PUT request.

    :param user: user to get token
    :param url: URL for the new :class:`Request` object.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """
    return basic_request('put', url, user=user, json=json, **kwargs)


def patch(url, json=None, user=None, **kwargs):
    """
    Sends a PATCH request.

    :param user: user to get token
    :param url: URL for the new :class:`Request` object.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """
    return basic_request('patch', url, user=user, json=json, **kwargs)


def delete(url, user=None, **kwargs):
    """
    Sends a DELETE request.

    :param user: user to get token
    :param url: URL for the new :class:`Request` object.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """
    return basic_request('delete', url, user=user, **kwargs)



import json
from enum import Enum

from api_manager.settings import ALMS_USER, LABS_USER
from api_manager.token import get_alms_token, get_labs_token
from api_manager.constants import Project


class HeadersTemplates(Enum):
    JSON = {'Content-type': 'application/json'}
    FORM_DATA = {'Content-type': 'multipart/form-data'}
    DEFAULT = {}


def alms_headers(user=None, with_auth=True, headers_type: HeadersTemplates = HeadersTemplates.DEFAULT):
    """
    :param headers_type:
    :param with_auth:
    :param user:
    :return:
    """
    if isinstance(user, dict):
        # for lru_cache user always should be hashable
        # overriding user credentials with insomnia
        user = json.dumps({**ALMS_USER, **user})

    if with_auth:
        token = get_alms_token(user)
        return {**headers_type.value, 'Authorization': f'Bearer {token}'}

    return headers_type.value


def labs_headers(user=None, with_auth=True, headers_type: HeadersTemplates = HeadersTemplates.DEFAULT):
    """
    :param headers_type:
    :param with_auth:
    :param user:
    :return:
    """
    if isinstance(user, dict):
        # for lru_cache user always should be hashable
        # overriding user credentials with insomnia
        user = json.dumps({**LABS_USER, **user})

    if with_auth:
        token = get_labs_token(user)
        return {**headers_type.value, 'Authorization': f'Token {token}'}

    return headers_type.value


PROJECT_HEADERS = {
    Project.LABS: labs_headers,
    Project.ALMS: alms_headers
}



import json
import logging
import re
from http import HTTPStatus
from typing import Optional

import allure
from requests import Response
from waiting import wait as wait_lib

from api_manager.constants import Entities, HttpActions, APIState
from api_manager.exceptions import NotConfiguredException
from api_manager.methods import get

ASYNC_COMMAND_IGNORE_CODES = [HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN, HTTPStatus.INTERNAL_SERVER_ERROR]


def wait(*args, **kwargs):
    """
    Wrapping 'wait()' method of 'waiting' library with default parameter values.
    WebDriverException is ignored in the expected exceptions by default.
    """
    from api_manager.settings import WAIT_TIMEOUT

    if WAIT_TIMEOUT is None:
        raise NotConfiguredException('You should define "WAIT_TIMEOUT" setting in your settings file')

    kwargs.setdefault('sleep_seconds', (1, None))
    kwargs.setdefault('timeout_seconds', WAIT_TIMEOUT)

    return wait_lib(*args, **kwargs)


def parse_uuid_from_string(string: str) -> Optional[str]:
    """
    :param string: Any string
    :return: If string contains uuid, then return uuid else None

    Example:
        some_url = 'https://some.domain.com/api/v1/create-activities/fdee2394-2aec-4af7-bb97-a6bdab88c9e5'
        parse_uuid_from_string(some_url) -> 'fdee2394-2aec-4af7-bb97-a6bdab88c9e5'
    """
    result = re.search(r'[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}', string)
    if result is None:
        return

    return result.group()


def get_request(url, message, user=None):
    """Wrapper to check lazy request"""
    from api_manager.settings import ALMS_USERS_API  # no qa

    if ALMS_USERS_API is None:
        raise NotConfiguredException('You should define "ALMS_USERS_API" setting in your settings file')

    with allure.step(message):
        return get(ALMS_USERS_API + url, user=user)


def async_response(response: Response, entity: Entities, check_message: Optional[str] = None, **kwargs) -> Response:
    """
    Wrapper around async request processing. Mostly to abstract autotest from this async logic
    and focus only on business logic.

    Flow:
    1. Creating entity and getting command endpoint
    {
        "id": "55c71154-bfcc-46c7-8747-50354bdcdd45",
        "url": "/api/v1/some-url/55c71154-bfcc-46c7-8747-50354bdcdd45", <- here
        "message": "Request accepted for processing"
    }
    2. Go to "/api/v1/some-url/55c71154-bfcc-46c7-8747-50354bdcdd45" and wait until command completed
    3. If command success, then will be following response
    {
        "id": "c5a0ad21-c157-47ba-9ad0-35925a4f3596",
        "entityId": "3fa85f64-5717-4562-b3fc-2c963f66afa6",
        "created": "2021-12-20T12:46:56.081087Z",
        "completed": {
            "url": "/api/v1/activities/3fa85f64-5717-4562-b3fc-2c963f66afa6",
            "completed": "2021-12-20T12:46:56.104712Z",
            "state": 1,
            "errors": "null"
        }
    }
    4. Go to "/api/v1/activities/3fa85f64-5717-4562-b3fc-2c963f66afa6" and get created entity
    """
    url = response.json().get('url')

    # if no url returned in response then probable it is not async request
    if url is None:
        return response

    action = HttpActions[response.request.method].value  # action depends on request method
    safe_entity = Entities.to_human_readable(entity).lower()  # entity name to process

    wait_message = check_message or f'Checking while {safe_entity} successfully {action}'

    # ensure that we can check this command, if not, it makes no sense to go further
    command_response = get_request(url, wait_message, **kwargs)
    if command_response.status_code in ASYNC_COMMAND_IGNORE_CODES:
        return command_response

    # waiting for async command to be completed on the server
    wait(lambda: get_request(url, wait_message, **kwargs).json()['completed'], waiting_for=wait_message)

    # getting completed command response
    command_response = get_request(url, wait_message, **kwargs)

    try:
        command_response_json = command_response.json()
    except json.decoder.JSONDecodeError as error:
        # if we did not receive json then it makes no sense to go further
        logging.error(error)
        raise AssertionError(f'Unable to get json from {command_response.content}')

    # if command completion failed then it makes no sense to go further
    if command_response_json['completed']['state'] == APIState.FAILED.value:
        # returning failed response
        return command_response

    try:
        entity_url = command_response_json['completed']['url']
    except KeyError as error:
        # if we could not get url from completed, for example completed always null
        # then it makes no sense to go further
        logging.error(error)
        raise AssertionError(f'Async command not success. Unable to get entity url from {command_response_json}')

    entity_message = f'Getting {safe_entity} with id "{parse_uuid_from_string(entity_url)}"'
    entity = get_request(entity_url, entity_message, **kwargs)
    entity.request.method = response.request.method  # override to actual request method
    return entity



import re
from enum import Enum
from typing import List


class Project(Enum):
    ALMS = 'alms'
    LABS = 'labs'


class APIState(Enum):
    """
    API response state

    Some of responses contains response state:
    1 - means success
    2 - means error
    """
    SUCCESS = 1
    FAILED = 2


class HttpActions(Enum):
    POST = 'created'
    PUT = 'updated'
    PATCH = 'updated'
    DELETE = 'deleted'


class Entities(Enum):
    ACTIVITY = 'Activity'
    GROUP = 'Group'
    GROUP_USER = 'GroupUser'
    LMS_USER = 'Lms user'
    OAUTH1_CREDENTIALS = 'Oauth1 credentials'
    OBJECTIVE = 'Objective'
    OBJECTIVE_ACCESS = 'Objective access'
    OBJECTIVE_WORKFLOW = 'Objective workflow'
    PERMISSION = 'Permission'
    ROLE = 'Role'
    ROLE_PATTERN = 'Role pattern'
    ROLE_PATTERN_PERMISSION = 'Role pattern permission'
    USER_ROLE = 'User role'
    USER = 'User'
    TENANT = 'Tenant'
    TENANT_SETTING = 'TenantSetting'
    RESOURCE_LIBRARIES = 'Resource libraries'
    GRADING_SCALE = 'Grading scale'

    @classmethod
    def to_human_readable(cls, entity: 'Entities') -> str:
        entity_as_list: List[str] = re.findall('[A-Z][^A-Z]*', entity.value)
        return ' '.join(entity_as_list)



import json
from functools import lru_cache

import requests

from api_manager.exceptions import NotConfiguredException


@lru_cache(maxsize=None)
def get_alms_token(user=None):
    """
    :param user:
    :return:
    """
    from api_manager.settings import ALMS_IDENTITY_SERVER, ALMS_USER  # no qa

    if ALMS_IDENTITY_SERVER is None:
        raise NotConfiguredException('You should define "ALMS_IDENTITY_SERVER" setting in your settings file')

    if ALMS_USER is None:
        raise NotConfiguredException('You should define "ALMS_USER" setting in your settings file')

    if isinstance(user, str):
        user = json.loads(user)

    response = requests.post(ALMS_IDENTITY_SERVER + '/connect/token', data=user or ALMS_USER, verify=False)
    json_response = response.json()
    return json_response.get('access_token')


@lru_cache(maxsize=None)
def get_labs_token(user=None):
    from api_manager.settings import LABS_API, LABS_USER  # no qa

    if LABS_API is None:
        raise NotConfiguredException('You should define "ALMS_IDENTITY_SERVER" setting in your settings file')

    if LABS_USER is None:
        raise NotConfiguredException('You should define "ALMS_USER" setting in your settings file')

    if isinstance(user, str):
        user = json.loads(user)

    response = requests.post(LABS_API + '/auth/login/', data=user or LABS_USER)
    json_response = response.json()
    return json_response.get('token')



import functools
from typing import Optional, List

from requests import Response

from api_manager.async_response.utils import ASYNC_COMMAND_IGNORE_CODES, async_response
from api_manager.constants import Entities


def lazy_request(entity: Entities, check_message: Optional[str] = None, ignore_codes: List[int] = None):
    """
    :param ignore_codes: List of status codes which will be ignored to check completion
    :param entity: Entity name
    :param check_message: Overrides default check message
    :return:

    Decorator which used for requests that are not returning
    response directly. Usually such response return 202 status code,
    this means request was "accepted" for processing but not
    completed yet. This decorator expects api return something like:
    {
        "id": <some_uuid>,
        "url": <url_where_we_should_check_status>,
        "message": <some_message>
    }
    Then decorator takes "url" param and checking it for completion and
    only after request completed, returns response.

    Example usage:

    @lazy_request(message='Some allure message', is_lazy=False)
    def my_async_request():
        ...
    """

    def inner(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            response: Response = func(*args, **kwargs)

            if response.status_code in (ignore_codes or ASYNC_COMMAND_IGNORE_CODES):
                return response

            if kwargs.get('is_lazy'):
                return response

            return async_response(response, entity, check_message, **kwargs)

        return wrapper

    return inner



from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ['requests', 'allure-python-commons', 'waiting']

setup(
    name="api_manager",
    version="0.2.0",
    author="Nikita Filonov",
    author_email="filonov.nikitkaa@gmail.com",
    description="Api Manager",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/Nikita-Filonov/models_manager",
    packages=find_packages(),
    install_requires=requirements,
    package_data={'api_manager': ['api_manager/templates/*']},
    include_package_data=False
)



import json
import logging
import os
import uuid
from shlex import quote

import allure


def logging_response(response):
    """
    :param response:
    :return:
    """
    if not hasattr(response, 'request'):
        return response

    try:
        message = response_log_message(response)
    except Exception as error:
        logging.error(f'Logger was unable to attach log. Error happened {error}')
        return

    with open(f'{uuid.uuid4()}.html', 'w') as file:
        file.write(message)

    try:
        allure.attach.file(
            file.name,
            name='Response content',
            attachment_type=allure.attachment_type.HTML
        )
    except KeyError:
        pass

    os.remove(file.name)

    return response


def to_curl(request, compressed=False, verify=True) -> str:
    """
    Returns string with curl command by provided request object

    Parameters
    ----------
    compressed : bool
        If `True` then `--compressed` argument will be added to result

    :param verify:
    :param compressed:
    :param request:
    """
    parts = [
        ('curl', None),
        ('-X', request.method),
    ]

    for k, v in sorted(request.headers.items()):
        parts += [('-H', '{0}: {1}'.format(k, v))]

    if request.body:
        body = request.body
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except UnicodeDecodeError:
                body = {}

        parts += [('-d', body)]

    if compressed:
        parts += [('--compressed', None)]

    if not verify:
        parts += [('--insecure', None)]

    parts += [(None, request.url)]

    flat_parts = []
    for k, v in parts:
        if k:
            flat_parts.append(quote(k))
        if v:
            flat_parts.append(quote(v))

    return ' '.join(flat_parts)


def response_log_message(response):
    """
    :param response:
    :return:
    """
    try:
        response_content = response.json()
    except json.JSONDecodeError:
        response_content = response.content

    payload = {
        'request_url': response.request.url,
        'request_method': response.request.method,
        'request_headers': response.request.headers,
        'request_content': response.request.body,
        'response_headers': response.headers,
        'cookies': response.cookies,
        'response_content': response_content,
        'status_code': response.status_code,
        'reason': response.reason,
        'curl': to_curl(response.request, verify=False)
    }

    with open('./templates/log_template.html', 'r') as template:
        return str(template.read()).format(**payload)
